----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    11:55:32 10/18/2010 
-- Design Name: 
-- Module Name:    Delay - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Delay is
	generic 
				(
				system_speed 	: integer := 50000000;						-- System Speed in Hz
				bitrate 			: integer := 1000000;						-- Bitrate = 1Mbit/s (spter variabel)
				time_delay 		: integer := 2000
				);
	port
		(
		RESET							: in std_logic;							-- asynchroner Reset
		clk_i							: in std_logic;							-- Clock fr Receive (50MHz)
		start							: in std_logic;
		valid							: in std_logic;
		valid_first_bitwise		: in std_logic;
		valid_scnd_bitwise		: in std_logic;
		valid_delay					: out std_logic := '0';
	--	valid_bitwise				: in std_logic;
		bitwise_dat1_delay_in	: in std_logic_vector(15 downto 0) := (others => '0');-- Datenvektor der jedes einzelne Bit ausgibt
		bitwise_dat2_delay_in	: in std_logic_vector(15 downto 0) := (others => '0');-- Datenvektor der jedes einzelne Bit ausgibt
		dat_delay_in				: in std_logic_vector(15 downto 0) := (others => '0');-- Datenvektor der jedes einzelne Bit ausgibt
		dat_delay_out				: out std_logic_vector(15 downto 0) := (others => '0')
		);
end Delay;

architecture Behavioral of Delay is

constant max_time_counter 	: natural := (time_delay / 100) * (system_speed / 10000000);
constant max_counter			: natural := system_speed / bitrate;

type STATE_TYPE is 
		(
			INIT,
			TIMER_START,
			SEND_FIRST_BITWISE,
			SEND_SCND_BITWISE,
			SEND_DATA
		);
		
signal STATE						: STATE_TYPE:= INIT;
signal time_counter				: natural range 0 to max_time_counter := 0;
signal bitrate_counter			: natural range 0 to max_counter := 0;
signal bit_counter				: natural range 0 to 15 := 0;
signal delay_bit_counter		: natural range 0 to 31;
signal shift_register			: std_logic_vector(15 downto 0) := (others => '0');
signal shift_register_tmp		: std_logic_vector(15 downto 0) := (others => '0');
signal delay_bit_counter_save	: natural range 0 to 15;

begin

Delay : process(clk_i)
	begin
		if rising_edge(clk_i) then
			if RESET = '1' then
				STATE <= INIT;
			else
				case STATE is
					when INIT =>
						if start = '1' then
							STATE <= TIMER_START;
							time_counter <= max_time_counter - 5;
							bitrate_counter <= max_counter - 1;
							bit_counter <= 15;
							delay_bit_counter <= 0;
						end if;
					
					when TIMER_START =>
						if time_counter = 0 then
							if valid_first_bitwise = '1' or valid_scnd_bitwise = '1' then	
								STATE <= SEND_FIRST_BITWISE;
								if valid_first_bitwise = '1' then											--1. Bit erst speichern
									shift_register(0) <= bitwise_dat1_delay_in(delay_bit_counter);	--wenn Daten gltig
								end if;																				--
								bit_counter <= 15;
								if valid_scnd_bitwise = '1' then  					--Wenn Timer grer als 1. Datenwort
									delay_bit_counter <= delay_bit_counter_save;	--wird delay_counter erneut auf 31 
								end if;														--gesetzt
							else
								STATE <= SEND_DATA;
							end if;
							valid_delay <= '1';
							bitrate_counter <= max_counter - 1;
						else
							time_counter <= time_counter - 1;
							if valid_first_bitwise = '1' or valid_scnd_bitwise = '1' then
								if bitrate_counter = 0 then
									if bit_counter = 0 then
										delay_bit_counter <= 0;
									else
										bit_counter <= bit_counter - 1;
										delay_bit_counter <= delay_bit_counter + 1;
										delay_bit_counter_save <= delay_bit_counter_save + 1;
										bitrate_counter <= max_counter - 1;
									end if;
								else
									bitrate_counter <= bitrate_counter - 1;
								end if;
							end if;
						end if;
						
					when SEND_FIRST_BITWISE =>
						if bitrate_counter = 0 then
							if bit_counter = 0 then
								STATE <= SEND_SCND_BITWISE;
								bit_counter <= 15;
								bitrate_counter <= max_counter - 1;
								shift_register_tmp(0) <= bitwise_dat2_delay_in(delay_bit_counter);
							else
								shift_register <= shift_register(14 downto 0) & bitwise_dat1_delay_in(delay_bit_counter);
								bitrate_counter <= max_counter - 1;
								bit_counter <= bit_counter - 1;
							end if;
						else
							if delay_bit_counter = bit_counter then					--Zhlt delay bit runter,
								delay_bit_counter <= delay_bit_counter - 1;			--damit die letzten Bits
								if bit_counter = 0 then										--eines Datenworts auch
									delay_bit_counter <= delay_bit_counter_save;		--bernommen werden. Danach
								end if;															--wieder auf Ursprungswert
							end if;																--
							dat_delay_out <= shift_register;
							bitrate_counter <= bitrate_counter - 1;
						end if;						
					
					when SEND_SCND_BITWISE =>
						if bitrate_counter = 0 then
							if bit_counter = 0 then
								STATE <= SEND_DATA;
								shift_register <=  shift_register(14 downto 0) & dat_delay_in(15);
								bit_counter <= 15;
							else
								shift_register_tmp <= shift_register_tmp(14 downto 0) & bitwise_dat2_delay_in(delay_bit_counter);
								bit_counter <= bit_counter - 1;
							end if;
							bitrate_counter <= max_counter - 1;
						else
							if delay_bit_counter = bit_counter then					--Zhlt delay bit runter,
								delay_bit_counter <= delay_bit_counter - 1;			--damit die letzten Bits
								if bit_counter = 0 then										--eines Datenworts auch
									delay_bit_counter <= delay_bit_counter_save;		--bernommen werden. Danach
								end if;															--wieder auf Ursprungswert
							end if;																--
							dat_delay_out <= shift_register_tmp;
							bitrate_counter <= bitrate_counter - 1;
						end if;
					
					when SEND_DATA =>
						if bitrate_counter = 0 then
							if bit_counter = 0 then
								bit_counter <= 15;
								--shift_register <= dat_delay_in;
								shift_register <=  shift_register(14 downto 0) & dat_delay_in(15);
							else
								bit_counter <= bit_counter - 1;
							end if;
							shift_register <=  shift_register(14 downto 0) & shift_register(15);
							bitrate_counter <= max_counter - 1;
						else
							dat_delay_out <= shift_register;
							bitrate_counter <= bitrate_counter - 1;
						end if;
--					
					
--						if bitrate_counter = 0 then
--							if bit_counter = 0 then
--								bit_counter <= 31;
--								bitrate_counter <= max_counter - 1;
--								dat_delay_out <= dat_delay_in;
--							else
--								--shift_register <= shift_register(30 downto 0) & dat_delay_in(delay_bit_counter);
--								bitrate_counter <= max_counter - 1;
--								bit_counter <= bit_counter - 1;
--							end if;
--						else
--							--dat_delay_out <= shift_register;
--							bitrate_counter <= bitrate_counter - 1;
--						end if;
				end case;		
			end if;
		end if;
	end process;
end Behavioral;



















