----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    15:24:36 10/11/2010 
-- Design Name: 
-- Module Name:    Receive - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity Receive is
	generic 
				(
				system_speed 	: integer := 50000000;						-- Clock = 50MHz
				bitrate 			: integer := 1000000							-- Bitrate = 1Mbit/s (spter variabel)
				);
	port 
				(
				RESET						: in std_logic;							-- asynchroner Reset
				clk_i						: in std_logic;							-- Clock fr Receive (50MHz)
				init_done_i				: in std_logic;							-- Initialisierungssignal des spteren DDR2-Rams
				timer_start				: out std_logic := '0';
				rx							: in std_logic;							-- Empfangsping
				valid_first_bitwise	: out std_logic := '0';					-- Datenvektor "bitwise_dat1_out" ist gltig
				valid_scnd_bitwise	: out std_logic := '0';					-- Datenvektor "bitwise_dat2_out" ist gltig
				valid_data				: out std_logic := '0';					-- Datenvektor "dat_out" ist gltig
				valid						: out std_logic := '0';					-- es liegen Daten an rx an
				bitwise_dat1_out		: out std_logic_vector(15 downto 0) := (others => '0');-- Datenvektor der jedes einzelne Bit ausgibt
				bitwise_dat2_out		: out std_logic_vector(15 downto 0) := (others => '0');-- Datenvektor der jedes einzelne Bit ausgibt
				dat_out					: out std_logic_vector(15 downto 0)	:= (others => '0')-- Datenvektor fr komplette 32 Bit
				); 
end Receive;

architecture Behavioral of Receive is

constant max_counter : natural := system_speed / bitrate;

type STATE_TYPE is 
		(
			INIT,
			WAIT_FOR_RX_START,
			WAIT_FOR_40NS,
			RECEIVE_BITS
		);

signal STATE					: STATE_TYPE := INIT;
signal bitrate_counter		: natural range 0 to max_counter := 0;						-- Downzhler zum ermitteln der Lnge eines Bits				
signal bit_counter			: natural range 0 to 15 := 0;									-- Zhlt, wieviel Bits gesendet wurden und fngt nach 32 von vorn an
signal shift_register		: std_logic_vector(15 downto 0) := (others => '0');	-- zum umwandeln der einzelnen Bits in einen Vektor
signal shift_register_tmp	: std_logic_vector(15 downto 0) := (others => '0');
signal only_two_bitwise		: natural range 0 to 2 := 0;									-- "Bitwise_dat_out" nur ein Datenwort gltig

begin

Receive : process(clk_i)
	begin
		if rising_edge(clk_i) then
			if RESET = '1' then
				STATE <= INIT;
				dat_out <= (others => '0');
			else
				case STATE is
					-- WARTEN AUF RAM-INITIALISIERUNG
					when INIT =>												
						if init_done_i = '1' then
							STATE <= WAIT_FOR_RX_START;
						end if;
					-- WARTEN BIS RX das erste mal 1 ist
					when WAIT_FOR_RX_START =>
						if rx = '1' then
							STATE <= WAIT_FOR_40NS;
							timer_start <= '1';
							bitrate_counter <= max_counter / max_counter - 1; -- setze "bitrate_counter" auf 1 (fr 40ns)
							only_two_bitwise <= 2;
							end if;		
					-- RX SOLL NICHT BEI FLANKENANSTIEFG ABGETASTET WERDEN
					when WAIT_FOR_40NS =>
						if bitrate_counter = 0 then
							STATE <= RECEIVE_BITS;
							bit_counter <= 15;
							bitrate_counter <= max_counter - 1;	-- "bitrate_counter" so gro wie 1 bit lang ist
							shift_register(0) <= rx;				-- sofortiges abtasten des 1. Bits
							valid <= '1';
							if only_two_bitwise = 2 then
								valid_first_bitwise <= '1';
								valid_scnd_bitwise <= '0';
							end if;
						else
							bitrate_counter <= bitrate_counter - 1;
						end if;
					-- BITS EMPFANGEN
					when RECEIVE_BITS =>
						if bitrate_counter = 0 then
							if bit_counter = 0 then
								dat_out <= shift_register;				-- bei nchsten Datenwort das gleiche wie bei "WAIT_FOR_40NS"
								bit_counter <= 15;
								bitrate_counter <= max_counter - 1;
								shift_register(0) <= rx;
								-- Schleife, damit nur das erste Datenwort auf den bitwise_dat_out vektor geschrieben wird
								if only_two_bitwise = 2 then
									valid_first_bitwise <= '0';
									valid_scnd_bitwise <= '1';
									valid_data <= '1';	
									only_two_bitwise <= only_two_bitwise - 1;
									shift_register_tmp(0) <= rx;
									--shift_register(0) <= rx;
								end if;	
								if only_two_bitwise = 1 then
									valid_first_bitwise <= '0';
									valid_scnd_bitwise <= '0';	
									only_two_bitwise <= only_two_bitwise - 1;
									--shift_register(0) <= rx;
								end if;
							else
								bit_counter <= bit_counter - 1;
								if only_two_bitwise = 1 then
									shift_register_tmp <= shift_register_tmp(14 downto 0) & rx;
									shift_register <= shift_register(14 downto 0) & rx;
								else
									shift_register <= shift_register(14 downto 0) & rx;
								end if;
							end if;
							bitrate_counter <= max_counter - 1;
						else
							bitrate_counter <= bitrate_counter - 1;
							if only_two_bitwise = 2 then
								bitwise_dat1_out <= shift_register;
							end if;
							if only_two_bitwise = 1 then
								bitwise_dat2_out <= shift_register_tmp;
							end if;
							
						end if;
						if only_two_bitwise = 2 then
							valid_first_bitwise <= '1';
							valid_scnd_bitwise <= '0';
						end if;
				end case;
			end if;
		end if;
	end process;
end Behavioral;












